//package mariana;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Formatter;
import java.util.Random;

public class CategoryMatrix extends DocumentsInOut 
{
	private int[][] catMatrix;
	private String categoryFile;
	private int docCount;
	private int catTotal;
	private String[] catNames;
	private Double[] usageSplit = {0.5, 0.25, 0.25};
	private int[] usage;
	private int[] trainDocs;
	private int[] valDocs;
	private int[] testDocs;
	private int[][] catCount;
	private int[][] trainCatCounts;
	private int[][] valCatCounts;
	private int[][] tstCatCounts;
	protected DocumentsInOut[] dInOut;



	public  CategoryMatrix()
	{
		// HI
	}

	public  CategoryMatrix(String fileName)
	{
		try {
			this.readCategories(fileName);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void readCategories(String fileName) throws IOException
	{
		categoryFile = fileName;
		docCount = 0;
		int len = 0;
		byte bytArr[] = null;
		try {

			Tic readTime = new Tic();
			len = (int)(new File(categoryFile).length());
			FileInputStream fis = new FileInputStream(categoryFile);

			bytArr = new byte[len];
			fis.read(bytArr);
			fis.close();
		} catch (FileNotFoundException e) {
			System.err.print("ERROR: Cant find file " + categoryFile);
		}
		catch (IOException e) { 
			System.err.print("ERROR: Failed to open file " + categoryFile);
		}
		// count the lines and the categories
		for (int i = 0; i < len; i++) {
			if (bytArr[i] == '\n') {
//				newLines.add(i);
				docCount++;
			}
		}
		if ( bytArr[len-1] != '\n') docCount++; // if the last line does not have a new line


		// convert the bytarr to a reader.
		BufferedReader reader = new BufferedReader(
				new StringReader(new String(bytArr)));
		// count the number of categories
		String line = null;
		String[] firstLine = null;

		try { 
			line = reader.readLine();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		firstLine = line.split(",");
		catTotal = firstLine.length;

		// set the category sizes
		catNames = new String[catTotal];

		// check if the first line are the category names.
		if(firstLine[0].startsWith("#"))
		{
			for(int i = 0; i < catTotal; i++)
				catNames[i] = firstLine[i].replace("#", ""); 
			--docCount;
			line = reader.readLine();
		}
		else
			for(int i = 0; i < catTotal; i++)
				catNames[i] = "Cat_" + ( i+1 ); 



		if(catTotal == 0) 
		{
			System.out.println("Zero categories");
			System.exit(-1);
		}

		// allocate memory
		catMatrix = new int[docCount][catTotal];

		// read in the category information
		try {
			int j = 0;
			while (line != null && j < docCount)
			{
				firstLine = line.split(",");
				if(firstLine.length != catTotal)
				{
					System.err.println("Category Counts do not match on line"
							+ (j+1));
					System.exit(-1);
				}
				for (int i = 0; i < catTotal; i++)
				{
					int a = Integer.valueOf(firstLine[i]);
					if(a == 1) catMatrix[j][i] = 1;
					else if (a == -1) catMatrix[j][i] = -1;
					else 
					{
						//System.err.println("Ambiguous Category Label: Line "
						//	+ (j+1) + " Column " + (i+1));
						if(a <= 0) a = -1;
						else a = 1;
						//System.err.println("changing label to " + a);
						//System.exit(-1);
					}

				}
				j++;
				line = reader.readLine();
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		dInOut = new DocumentsInOut[catTotal];
		for(int i = 0; i < catTotal; i++) dInOut[i] = new DocumentsInOut();



	}

	public void setUsage()
	{

		int tst, trn, val;
		trn = (int) Math.floor(this.usageSplit[0]*(double)docCount);
		tst = (int) Math.floor(this.usageSplit[1]*(double)docCount);
		val = docCount - trn - tst;

		trainDocs = new int[trn];
		valDocs = new int[val];
		testDocs = new int[tst];

		//System.out.println(trn + " " + val + " " + tst);
		// build a vector the size of all docs and assign usage
		int[] tmpUsage = new int[docCount];
		int i = 0;
		for(i = 0; i < trn; i++) tmpUsage[i] = 999;
		for(i = trn; i < trn + val; i++) tmpUsage[i] = -888;
		for(i = trn + val; i < docCount; i++) tmpUsage[i] = 7537;

		if(i != docCount) System.err.println("check the number of docs");

		// randomly sort this array
		Random generator = new Random(  );
		for (i = docCount - 1; i > 0; i--)
		{
			//choose a random position between i and n-1
			int which = generator.nextInt(i);
			//swap permutation[i] and permutation[which]
			int tmp = tmpUsage[i];
			tmpUsage[i] = tmpUsage[which];
			tmpUsage[which] = tmp;
		}


		int trnCnt = 0;
		int valCnt = 0;
		int tstCnt = 0;

		for (i = 0; i < docCount; i++)
		{
			// the bow starts at document 1 not zero.
			if(tmpUsage[i] == 999)
				trainDocs[trnCnt++] = i + 1;
			if (tmpUsage[i] == -888)
				valDocs[valCnt++] = i + 1;
			if (tmpUsage[i] == 7537)
				testDocs[tstCnt++] = i + 1;

			if(trnCnt > trn || valCnt > val || tstCnt > tst)
			{
				System.err.println("the Counts are not correct");
				System.err.println(i + " " + trnCnt +" " + valCnt + " " + tstCnt + " ");
				System.exit(-1);
			}

		}
		usage = tmpUsage;
		// cout how may doc in each category 
		trainCatCounts = new int[2][catTotal];
		valCatCounts = new int[2][catTotal];
		tstCatCounts = new int[2][catTotal];

		ArrayList<Integer> inDocs = new ArrayList<Integer>();
		ArrayList<Integer> outDocs = new ArrayList<Integer>();
		ArrayList<Integer> valIn = new ArrayList<Integer>();
		ArrayList<Integer> valOut = new ArrayList<Integer>();
		ArrayList<Integer> testIn = new ArrayList<Integer>();
		ArrayList<Integer> testOut = new ArrayList<Integer>();
		for (i = 0; i < catTotal; i++)
		{
			trainCatCounts[0][i] = 0;
			trainCatCounts[1][i] = 0;
			valCatCounts[0][i] = 0;
			valCatCounts[1][i] = 0;
			tstCatCounts[0][i] = 0;
			tstCatCounts[1][i] = 0;


			for (int j = 0; j < docCount; j++)
			{
				if(usage[j] == 999)
				{
					if(catMatrix[j][i] == 1)
					{
						trainCatCounts[0][i]++;
						inDocs.add(j+1);// the bow is starts at 1
					}
					else
					{
						trainCatCounts[1][i]++;
						outDocs.add(j+1);
					}
				}
				if(usage[j] == -888)
				{
					if(catMatrix[j][i] == 1)
					{
						valCatCounts[0][i]++;
						valIn.add(j+1);
					}
					else
					{
						valCatCounts[1][i]++;
						valOut.add(j+1);
					}
				}
				if(usage[j] == 7537)
				{
					if(catMatrix[j][i] == 1)
					{ 
						tstCatCounts[0][i]++;
						testIn.add(j+1);
					}
					else
					{
						tstCatCounts[1][i]++;
						testOut.add(j+1);
					}
				}
			}

			//Stupid conversions
			int a[] = new int[inDocs.size()];
			int b[] = new int[outDocs.size()];
			for(int k = 0; k < inDocs.size(); k++) a[k] = inDocs.get(k).intValue(); 
			for(int k = 0; k < outDocs.size(); k++) b[k] = outDocs.get(k).intValue();
			//System.out.printf("a and b length %d\t%s\n", a.length, b.length);
			dInOut[i].fill(a, b);
			dInOut[i].fillVal(valIn, valOut);
			dInOut[i].fillTest(testIn, testOut);
			inDocs.clear();
			outDocs.clear();
			valIn.clear();
			valOut.clear();
			testIn.clear();
			testOut.clear();

		}	
//		for (int i = 0; i < this.testDocs.length; i++)
//		System.out.println(i + " " + this.testDocs[i]);
	}

	// 
	public void printCatSplits()
	{

		Formatter output = new Formatter();
		output.format("trn\n");
		for(int j = 0; j < 2; j++){
			for(int i = 0; i < catTotal; i++){
				output.format("%d\t",trainCatCounts[j][i]);
				//output += tmp;//Integer.toString(trainCatCounts[j][i]) + "\t";
			}
			output.format("\n");// += "\n";
		}
		output.format("val\n");
		for(int j = 0; j < 2; j++){
			for(int i = 0; i < catTotal; i++)
				output.format("%d\t",valCatCounts[j][i]) ;
			output.format("\n");
		}
		output.format("tst\n");
		for(int j = 0; j < 2; j++){
			for(int i = 0; i < catTotal; i++)
				output.format("%d\t", tstCatCounts[j][i]);
			output.format("\n");
		}
		System.out.println(output);

	}

	public int[][] getDocList(int c, int number, double per)
	{
		int[] docList = dInOut[c].getDocList(number, per);
		int [][] docCatList = new int[docList.length][2];
		for(int i = 0; i < docList.length; i++)
		{
			docCatList[i][0] = docList[i];
			docCatList[i][1] = catMatrix[docList[i]][c];
			//System.out.println(Arrays.toString(docCatList[i]));
		}
		return docCatList;
	}

	public int[] getSize()
	{
		int size[] = new int[2];
		size[0] = docCount;
		size[1] = catTotal;

		return size;
	}

	public String catName(int c)
	{
		return catNames[c];
	}

	public int docCount()
	{
		return docCount;
	}

	public int[] getCatVector(int c)
	{
		int catVector[] = new int[docCount];

		for(int i = 0; i < docCount; i++)
		{
			catVector[i] = catMatrix[i][c];
		}

		return catVector;
	}

	public String getCatString()
	{
		StringBuffer output = new StringBuffer("");

		for (int i = 0; i < docCount; i++)
		{
			output.append(Arrays.toString(catMatrix[i]));
			output.append("\n");
		}
		return output.toString();
	}

	public String getDocUsage()
	{
		StringBuffer output = new StringBuffer("");

		output.append(Arrays.toString(trainDocs));
		output.append("\n");
		output.append(Arrays.toString(valDocs));
		output.append("\n");
		output.append(Arrays.toString(testDocs));
		output.append("\n");
		
		return output.toString();
	}
	
	public String getCatUsage(int c)
	{
		return dInOut[c].getFullUsage();
	}
	
	public String getInClass(int c)
	{
		StringBuffer output = new StringBuffer("");
		output.append(Arrays.toString(dInOut[c].getDocsIn()));
		
		return output.toString();
	}
	
	public String getOutClass(int c)
	{
		StringBuffer output = new StringBuffer("");
		output.append(Arrays.toString(dInOut[c].getDocsOut()));
		
		return output.toString();
	}
	
	public void loadDocsIn(String diString)
	{
		dInOut = new DocumentsInOut[1];
		dInOut[0].loadDocsIn(diString);
	}
}
