import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;


public class Collection
{
	protected BagOfWords bow;
	protected CategoryMatrix cats;
	protected TermStats terms;
	private int docCount;
	private int termCount;
	private int catCount;


	public Collection()
	{

	}

	public Collection(String textFile, String catFile, String categoryTmp) throws IOException
	{
		int tmpCat = Integer.parseInt(categoryTmp) - 1;
		String category = Integer.toString(tmpCat);
		if ( tmpCat < 0 )
		{
			System.err.printf("categories now use 1 based indexing\n");
			System.exit(1);
		}
		
		bow = new BagOfWords(textFile);
//		bow.build(textFile);
		cats = new CategoryMatrix(catFile);
		cats.setUsage();
//		cats.readCategories(catFile);

		// check to see if the sizes match
		int sizes[] = cats.getSize();
		docCount = sizes[0];
		catCount = sizes[1];

		if(bow.docCount() != docCount)
		{
			System.err.printf("Document Count Missmatch: %d text %d cat\n",
					bow.docCount(), docCount);
			//System.exit(-1);
		}
		termCount = bow.termCount();

		//terms = new TermStats[catCount];
		//for(int i = 0; i < catCount; i++)
		//{
		terms = new TermStats(bow, cats, Integer.parseInt(category));
		//}



	}
	
	public Collection(String textFile, String catFile, String thesFile, String categoryTmp) throws IOException
	{
		int tmpCat = Integer.parseInt(categoryTmp) - 1;
		String category = Integer.toString(tmpCat);
		if ( tmpCat < 0 )
		{
			System.err.printf("categories now use 1 based indexing\n");
			System.exit(1);
		}
		
		bow = new BagOfWords(textFile, thesFile);
//		bow.build(textFile);
		cats = new CategoryMatrix(catFile);
		cats.setUsage();
//		cats.readCategories(catFile);

		// check to see if the sizes match
		int sizes[] = cats.getSize();
		docCount = sizes[0];
		catCount = sizes[1];

		if(bow.docCount() != docCount)
		{
			System.err.printf("Document Count Missmatch: %d text %d cat\n",
					bow.docCount(), docCount);
			//System.exit(-1);
		}
		termCount = bow.termCount();

		//changed by Kanishka 05/14/2009
		//terms = new TermStats[catCount];
		//for(int i = 0; i < catCount; i++)
		//{
		terms = new TermStats(bow, cats, Integer.parseInt(category));
		//}

		
	}

	public void printTermStats(String fileName, String categoryTmp) throws IOException
	{
		int tmpCat = Integer.parseInt(categoryTmp) - 1;
		String category = Integer.toString(tmpCat);
		if ( tmpCat < 0 )
		{
			System.err.printf("categories now use 1 based indexing\n");
			System.exit(1);
		}
		
		BufferedWriter writer = new BufferedWriter(new FileWriter(fileName));

		String header = "Terms, ID, ";
		//for(int i = 0; i < catCount; i++)
		//{
		header += terms.getHeader();
		if(Integer.parseInt(category)<catCount-1) 
			header += ", ";
		else 
			header += "\n";
		//}
		writer.write(header);
		//writer.write("\n");

		double tmpStats[] = null;
		for(int i = 0; i < termCount; i++)
		{
			String statString = "";
			statString += bow.getTerm(i+1);
			statString += String.format(", %d", i+1);
			tmpStats = terms.getTermStat(i);
			for(int k = 0; k < tmpStats.length; k++)
				statString += String.format(", %g", tmpStats[k]);
			statString += "\n";

			writer.write(statString);
			if(i%1000 == 0) 
				System.out.printf("String build %d of %d\n", i, termCount);
		}

		writer.close();
	}

	public void saveCollection(String outputFile, String categoryTmp)
	{
		int tmpCat = Integer.parseInt(categoryTmp) - 1;
		String category = Integer.toString(tmpCat);
		if ( tmpCat < 0 )
		{
			System.err.printf("categories now use 1 based indexing\n");
			System.exit(1);
		}
		
		// set up the zipfile to save
		int level = 9;
		int start = 1;

		FileOutputStream fout = null;
		try {
			fout = new FileOutputStream(outputFile);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		ZipOutputStream zout = new ZipOutputStream(fout);
		zout.setLevel(level);

		// save the general information and parts that apply
		// to all categories.

		/****** general info ******/
		String fn = "general_info.txt";
		ZipEntry ze = new ZipEntry(fn);
		try {
			zout.putNextEntry(ze);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		String output = bow.getInfo();
		try {
			zout.write(output.getBytes(), 0, output.length());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		/****** category matrix ******/
		fn = "category_matrix.csv";

		ze = new ZipEntry(fn);
		try {
			zout.putNextEntry(ze);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		output = cats.getCatString();
		try {
			zout.write(output.getBytes(), 0, output.length());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		/***** thesaurus *****/
		if(bow.thes != null)
		{
			fn = "thesaurus.txt";

			ze = new ZipEntry(fn);
			try {
				zout.putNextEntry(ze);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			output = bow.thes.getThesaurus();
			try {
				zout.write(output.getBytes(), 0, output.length());
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		/***** terms *****/
		fn = "terms.txt";

		ze = new ZipEntry(fn);
		try {
			zout.putNextEntry(ze);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		output = bow.getTerms();
		try {
			zout.write(output.getBytes(), 0, output.length());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		/***** bow *****/
		fn = "bow.csv";

		ze = new ZipEntry(fn);
		try {
			zout.putNextEntry(ze);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		output = bow.getBowString();
		try {
			zout.write(output.getBytes(), 0, output.length());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		/***** usage *****/
		fn = "usage.csv";

		ze = new ZipEntry(fn);
		try {
			zout.putNextEntry(ze);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		output = cats.getDocUsage();
		try {
			zout.write(output.getBytes(), 0, output.length());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		/***** Save the category information ****/
		/* Each file starts with cat and the category number: cat_#
		 * There are one of the following for each category
		 * 
		 * 		inClass
		 * 		outClass
		 * 		termRank
		 * 		termStats
		 * 		termNorm
		 * 
		 * 
		 */

		String fileTypes[] = {
				"inClass",
				"outClass",
				"termRank",
				"termStats",
				"termNorm",
				"fullUsage"};

		for (int j = 0; j < fileTypes.length; j++)
		{
			// generate the name
			fn = "cat_" + Integer.toString(Integer.parseInt(category) + 1)
			+ "_" + fileTypes[j] + ".csv";

			ze = new ZipEntry(fn);
			try 
			{
					
				zout.putNextEntry(ze);
				} 
			catch (IOException e) 
			{			
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			switch (j)
			{
				case 0:	output = cats.getInClass(Integer.parseInt(category)); break;
				case 1:	output = cats.getOutClass(Integer.parseInt(category)); break;
				case 2:	output = terms.getTermRank(); break;
				case 3: output = terms.getTermStatsString(); break;
				case 4: output = terms.getTermNorm(); break;
				case 5: output = cats.getCatUsage(Integer.parseInt(category)); break;
			}
			try 
			{
				zout.write(output.getBytes(), 0, output.length());
			} 
			catch (IOException e) 
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		try 
		{
			zout.close();
		} 
		catch (IOException e) 
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}



	public void readProblem(String inputFile, int c) 
	{

		/******check if everything is empty******/
		if(bow != null)
		{
			System.err.println("Clear BOW before loading a problem");
			System.exit(-1);
		}
		
		/******build the list of files to read******/
		


		try
		{
			FileInputStream fis = new FileInputStream(inputFile);
			ZipInputStream zis = new ZipInputStream(fis);
			ZipEntry ze;

			while((ze=zis.getNextEntry())!=null)
			{
				
			}
		}
		catch (FileNotFoundException e)
		{

		}
		catch (IOException e)
		{

		}
	}

}
