import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;


public class Predict {

	private String model;
	private Thesaurus thes;
	private TreeMap<Integer, Double> norm;
	private TreeMap<Integer, String> termID;
	private TreeMap<String, Integer> termMap;
	private String bowIn;

	public void loadModel(String modelFile)
	{
		String line;
		byte bytArr[] = null;
		int len = (int)(new File(modelFile).length());
		FileInputStream fis;
		try
		{
			fis = new FileInputStream(modelFile);

			bytArr = new byte[len];
			fis.read(bytArr);
			fis.close();

		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			System.err.print("Model File not found\n");
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			System.err.print("Problem loading model file\n");
			e.printStackTrace();
		}

		line = new String(bytArr);
		String[] mSplit = line.split("::LIBSVM::\n");

		// the first half contains the thesaurus, terms and norm
		String[] topHalf = mSplit[0].split("\n");

		// go through each line and parse 
		for(int i = 0; i < topHalf.length; i++)
		{
			String [] firstWord = topHalf[i].split(":", 2);
			//System.out.println(firstWord[0].trim());
			if(firstWord[0].equals("thesaurus"))
			{
				//System.out.println(firstWord[0].trim());

				String thesString = firstWord[1].trim().replaceAll(" ", "\n");
				if(!thesString.equals("")) thes.load(thesString);
			}
			if(firstWord[0].equals("terms"))
			{
				//System.out.println(firstWord[0].trim());

				termID = new TreeMap<Integer, String>();
				termMap = new TreeMap<String, Integer>();
				String[] terms = firstWord[1].trim().split(" ");
				for(int j = 0; j < terms.length; j++)
				{
					String[] b = terms[j].split(":");
					termID.put(Integer.valueOf(b[0]), b[1]);
					termMap.put(b[1], Integer.valueOf(b[0]));
				}
			}
			if(firstWord[0].equals("norm"))
			{
				//System.out.println(firstWord[1].trim());

				norm = new TreeMap<Integer, Double>();
				String[] norms = firstWord[1].trim().split(" ");
				for(int j = 0; j < norms.length; j++)
				{
					String[] b = norms[j].split(":");
					norm.put(Integer.valueOf(b[0]), Double.valueOf(b[1]));
				}
			}
		}

		model = mSplit[1];
		//System.out.println(termID.toString());
		//System.out.print(model); 


	}

	public void prepInput(String fileName)
	{
		// load the text

		int docCount = 0;
		byte bytArr[] = null;
		ArrayList<Integer> newLines = new ArrayList<Integer>();

		try 
		{

			int len = (int)(new File(fileName).length());
			FileInputStream fis = new FileInputStream(fileName);

			bytArr = new byte[len];
			fis.read(bytArr);
			fis.close();
			//System.out.println("Counting Docs: " + System.currentTimeMillis());
			for (int i = 0; i < len; i++) {
				if (bytArr[i] == '\n') {
					newLines.add(i);
					docCount++;
				}
			}
			if ( bytArr[len-1] != '\n') 
			{
				docCount++; // if the last line does not have a new line
				newLines.add(len);
			}
		
		}
		catch (FileNotFoundException e) {
			System.err.print("ERROR: Failed to open file " + fileName);
		}
		catch (IOException e) { 
			System.err.print("ERROR: Failed to open file " + fileName);
		}
		
		String inputTxt = new String(bytArr);
		
		String[] lines = inputTxt.split("\n");
		//System.out.print(lines.length);
		StringBuilder inputBow = new StringBuilder();
		for(int j = 0; j < lines.length; j++)
		{
			
			String strLine = lines[j];
			if (thes != null)
				strLine = thes.applyToString(strLine);
			else strLine = strLine.toUpperCase();
			strLine = strLine.replaceAll("[^A-Za-z]", " ");
			strLine = strLine.trim().replaceAll("\\b\\s{2,}\\b", " ");
			
			String[] termArray = strLine.split(" ");
			Map<Integer,Double> docVec1 = new HashMap<Integer,Double>();
			
			Object currentWordID = null;
			Object currentNorm = null;
			
			int i = 0;
			boolean indicator = false;
			
			while ( i < termArray.length)
			{
				//System.out.println(i + ":" + termArray[i] + ":" + uniqueTermCount);
				// the next if statement might or might not help.  Try on big file
				//if (termTree.contains(termArray[i])) 
				currentWordID = termMap.get(termArray[i]);
				
				if ((currentWordID) != null){
					//get the current count and add 1
					Double freq = docVec1.get((Integer)currentWordID);
					// get the norm
					currentNorm = norm.get((Integer)currentWordID);
					if(freq == null)
						docVec1.put((Integer)currentWordID, 1.0/(Double)currentNorm);
					else 
						docVec1.put((Integer)currentWordID, (freq + 1.0/(Double)currentNorm));
					
					indicator = true;
					
				}
				i++;

			}
			
			Map<Integer, Double> docVec = new TreeMap<Integer, Double>(docVec1);
			
			// normalize the values
			if(!indicator) inputBow.append("0 1:0\n");
			else
			{
				inputBow.append("0 " 
						+ docVec.toString().replaceAll("[\\{,}]", "").replaceAll("=", ":").trim()
						+ "\n");
				indicator = false;
			}
		}
		//System.out.print(inputBow.toString());
		bowIn = inputBow.toString();
	}
	
	public String predict()
	{
		MarianaInterface mar = new MarianaInterface();
		String results = mar.predict(model, bowIn);
		//System.out.println(results);
		return results;
	}
}
