import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.RandomAccessFile;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Map.Entry;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;


public class Problem 
{

	protected BagOfWords bow;
	protected CategoryMatrix cats;
	protected DocumentsInOut dInOut;
	protected TermStats terms;
	private int docCount;
	private int docUseCount;
	private int termCount;
	private int catCount;
	private int valid[];
	private int test[];
	private double norm[];
	private int rankIndx[];
	private double minPer;
	private String trainSVM;
	private String valSVM;
	private String tstSVM;
	
	protected MarianaInterface mar;



	public  Problem()
	{
		minPer = 0.1;
		docUseCount = 500;
		termCount = 500;
		//ystem.out.println("error: " + docUseCount);
	}

	public void readProblem(String zfile, int c)
	{

		/******check if everything is empty******/
		if(bow != null)
		{
			System.err.println("Clear BOW before loading a problem");
			System.exit(-1);
		}
		bow = new BagOfWords();
		terms = new TermStats();
		cats = new CategoryMatrix();
		dInOut = new DocumentsInOut();

		/******build the list of files to read******/
		HashMap<String, Integer> cmdMap = new HashMap<String, Integer>();
		cmdMap.put("bow.csv", 0);
		cmdMap.put("terms.txt", 1);
		cmdMap.put("thesaurus.csv", 2);
		cmdMap.put("cat_" + Integer.toString(c) + "_inClass.csv", 3);
		cmdMap.put("cat_" + Integer.toString(c) + "_outClass.csv", 4);
		cmdMap.put("cat_" + Integer.toString(c) + "_termNorm.csv", 5);
		cmdMap.put("cat_" + Integer.toString(c) + "_termRank.csv", 6);
		cmdMap.put("usage.csv", 7);
		cmdMap.put("cat_" + Integer.toString(c) + "_fullUsage.csv", 8);


		try {
			ZipFile zf = new ZipFile(zfile);
			Enumeration entries = zf.entries();

			BufferedReader input = new BufferedReader(new InputStreamReader(
					System.in));
			while (entries.hasMoreElements()) {
				ZipEntry ze = (ZipEntry) entries.nextElement();


				String fileName = ze.getName();
				Integer cmd = cmdMap.get(fileName);

				//System.out.println("zip entry " + fileName);
				//System.out.println("\tcommand " + cmd);

				if(cmd != null) 
				{
					long size = ze.getSize();
					if (size > 0) 
					{
						//System.out.println("Length is " + size);
						BufferedReader br = new BufferedReader(
								new InputStreamReader(zf.getInputStream(ze)));
						StringBuffer fullFile = new StringBuffer();
						String line;
						while ((line = br.readLine()) != null) 
						{
							fullFile.append(line);
							fullFile.append("\n");
						}
						br.close();
						//System.out.println(fullFile);
						switch(cmd)
						{
						case 0: bow.loadBow(fullFile.toString());break;
						case 1: bow.loadTerms(fullFile.toString()); break;
						case 2: bow.thes.load(fullFile.toString()); break;
						case 3: dInOut.loadDocsIn(fullFile.toString()); break;
						case 4: dInOut.loadDocsOut(fullFile.toString()); break;
						case 5: terms.loadTermNorm(fullFile.toString()); break;
						case 6: terms.loadTermRank(fullFile.toString()); break;
						case 7: this.loadUsage(fullFile.toString()); break;
						case 8: dInOut.loadFullUsage(fullFile.toString()); break;
						}
					}
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}



	}

	public void loadUsage(String usageString)
	{
		String[] useSplit = usageString.split("\n");

		for(int i = 1; i <= 2; i++)
		{
			useSplit[i] = useSplit[i].replaceAll("\\s+|\\[|\\]", "");
			//System.out.println(useSplit[i]);
			String[] d = useSplit[i].split(",");

			if(i == 1)
			{
				valid = new int[d.length];
				for(int j = 0; j < d.length; j++)
					valid[j] = Integer.valueOf(d[j]);
			}
			else
			{
				test = new int[d.length];
				for(int j = 0; j < d.length; j++)
					test[j] = Integer.valueOf(d[j]);
			}
		}
	}

	public void buildProblem(int numDoc, int numTerm)
	{
		docUseCount = numDoc;
		termCount = numTerm;
		// get the term list
		rankIndx = terms.reduceTerms(termCount);
		norm = new double[rankIndx.length];
//		System.out.println(Arrays.toString(termIndex));

		/* get the document list */ 
		// get the counts from the in and out sets
		int[] inOutCount = dInOut.countInOut();
		// the natural percentage
		double natPer = ((double)inOutCount[0]/((double)inOutCount[0] + (double)inOutCount[1]));
		double usePer = 0.1;
		//System.out.println("i/o " + inOutCount[0] + "/" + inOutCount[1] + " " + natPer);
		if(natPer < minPer || natPer > (1.0 - minPer))
		{
			// set the minimum per
			usePer = minPer;
		}
		else usePer = natPer;

		// count how many docs to use
		//int avalDocs = inOutCount[0] + inOutCount[1];
		int askDocIn = (int)Math.floor(usePer * (double)docUseCount);
		//System.out.println(docUseCount);
		int askDocOut = (int)Math.round((double)askDocIn*(-1.0 + 1.0/usePer));
		if( askDocIn > inOutCount[0])
			askDocIn = inOutCount[0];
		
		if(askDocOut > inOutCount[1])
			askDocOut = inOutCount[1];


		// get the indoc in a random order
//		int[] rpIn = randPerm(dInOut.getDocsIn());
		int[] rpIn = dInOut.getDocsIn();
		//System.out.println(Arrays.toString(rpIn));

		// get the outdocs in a random order
		int[] rpOut = randPerm(dInOut.getDocsOut());
		//System.out.println(Arrays.toString(rpOut));

		// build a string to be passed to libsvm
		StringBuilder trainStr = new StringBuilder();
		for(int i = 0; i < askDocIn; i++)
		{
			String s = ":";
			int[][] dv = bow.getDocVec(rpIn[i]);
			int j = 0, k = 0;
			trainStr.append("1");
			while(j < dv.length && k < rankIndx.length)
			{
				if(dv[j][0] == rankIndx[k])
				{
//					System.out.println(String.format("j=%d, k=%d, dv=%d, ti=%d, n=\n",
//							j, k, dv[j][0], termIndex[k]));
					double values = (double)dv[j][1];
					
					values /= terms.termNorm(rankIndx[k]);
					trainStr.append(String.format(" %d:%g", dv[j][0], values));
					j++;
					k++;
				}
				else if(dv[j][0] < rankIndx[k]) j++;
				else if(dv[j][0] > rankIndx[k]) k++;
				
				if(j >= dv.length || k > rankIndx.length) break;
			}
			trainStr.append("\n");
		}
		
		for(int i = 0; i < askDocOut; i++)
		{
			int[][] dv = bow.getDocVec(rpOut[i]);
			int j = 0, k = 0;
			trainStr.append("-1");
			while(j < dv.length && k < rankIndx.length)
			{
				if(dv[j][0] == rankIndx[k])
				{
//					System.out.println(String.format("j=%d, k=%d, dv=%d, ti=%d, n=\n",
//							j, k, dv[j][0], termIndex[k]));
					double values = (double)dv[j][1];
					values /= terms.termNorm(rankIndx[k]);
					trainStr.append(String.format(" %d:%g", dv[j][0], values));
					j++;
					k++;
				}
				else if(dv[j][0] < rankIndx[k]) j++;
				else if(dv[j][0] > rankIndx[k]) k++;
				
				if(j >= dv.length || k > rankIndx.length) break;
			}
			trainStr.append("\n");
		}

//		System.out.println(trainStr.toString());
		
		// build the validation set
		StringBuilder valStr = new StringBuilder();
		
		// first the valid in
		valid = dInOut.getValIn();
		for(int i = 0; i < valid.length; i++)
		{
			int[][] dv = bow.getDocVec(valid[i]);
			int j = 0, k = 0;
			//valStr.append(Integer.toString(valid[i]) + "  ");
			valStr.append("1");
			while(j < dv.length && k < rankIndx.length)
			{
				if(dv[j][0] == rankIndx[k])
				{
//					System.out.println(String.format("j=%d, k=%d, dv=%d, ti=%d, n=\n",
//							j, k, dv[j][0], termIndex[k]));
					double values = (double)dv[j][1];
					values /= terms.termNorm(rankIndx[k]);
					valStr.append(String.format(" %d:%g", dv[j][0], values));
					j++;
					k++;
				}
				else if(dv[j][0] < rankIndx[k]) j++;
				else if(dv[j][0] > rankIndx[k]) k++;
				
				if(j >= dv.length || k > rankIndx.length) break;
			}
			valStr.append("\n");
		}
		//valid out
		valid = dInOut.getValOut();
		for(int i = 0; i < valid.length; i++)
		{
			int[][] dv = bow.getDocVec(valid[i]);
			int j = 0, k = 0;
			//valStr.append(Integer.toString(valid[i]) + " ");
			valStr.append("-1");
			while(j < dv.length && k < rankIndx.length)
			{
				if(dv[j][0] == rankIndx[k])
				{
//					System.out.println(String.format("j=%d, k=%d, dv=%d, ti=%d, n=\n",
//							j, k, dv[j][0], termIndex[k]));
					double values = (double)dv[j][1];
					values /= terms.termNorm(rankIndx[k]);
					valStr.append(String.format(" %d:%g", dv[j][0], values));
					j++;
					k++;
				}
				else if(dv[j][0] < rankIndx[k]) j++;
				else if(dv[j][0] > rankIndx[k]) k++;
				
				if(j >= dv.length || k > rankIndx.length) break;
			}
			valStr.append("\n");
		}
//		System.out.println(valStr.toString());
		
		// build the test set
		StringBuilder testStr = new StringBuilder();
		
		// test in
		test = dInOut.getTestIn();
		for(int i = 0; i < test.length; i++)
		{
			int[][] dv = bow.getDocVec(test[i]);
			int j = 0, k = 0;
			testStr.append("1");
			while(j < dv.length && k < rankIndx.length)
			{
				if(dv[j][0] == rankIndx[k])
				{
//					System.out.println(String.format("i=%d, j=%d, k=%d, dv=%d, ti=%d, n=%d",
//							i, j, k, dv[j][0], rankIndx[k], rankIndx.length));
					double values = (double)dv[j][1];
					values /= terms.termNorm(rankIndx[k]);
					testStr.append(String.format(" %d:%g", dv[j][0], values));
					j++;
					k++;
				}
				else if(dv[j][0] < rankIndx[k]) j++;
				else if(dv[j][0] > rankIndx[k]) k++;
				
				if(j >= dv.length || k >= rankIndx.length) break;
			}
			testStr.append("\n");
		}
		//test out
		test = dInOut.getTestOut();
		for(int i = 0; i < test.length; i++)
		{
			int[][] dv = bow.getDocVec(test[i]);
			int j = 0, k = 0;
			testStr.append("-1");
			while(j < dv.length && k < rankIndx.length)
			{
				if(dv[j][0] == rankIndx[k])
				{
//					System.out.println(String.format("j=%d, k=%d, dv=%d, ti=%d, n=\n",
//							j, k, dv[j][0], termIndex[k]));
					double values = (double)dv[j][1];
					values /= terms.termNorm(rankIndx[k]);
					testStr.append(String.format(" %d:%g", dv[j][0], values));
					j++;
					k++;
				}
				else if(dv[j][0] < rankIndx[k]) j++;
				else if(dv[j][0] > rankIndx[k]) k++;
				
				if(j >= dv.length || k > rankIndx.length)
					{
						break;
					}
			}
			testStr.append("\n");
		}
		//System.out.println(testStr.toString());
		trainSVM = trainStr.toString();
		valSVM = valStr.toString();
		tstSVM = testStr.toString();
//		System.out.println("train\n" + trainSVM);
//		System.out.println("valid\n" + valSVM);
//		System.out.println("test\n" + tstSVM);
	}

	public int[] randPerm(int[] a)
	{
		// set up a hashmap of double and integers
		HashMap<Double, Integer> mp = new HashMap<Double, Integer>(a.length);
		// fill the hashmap
		for(int i = 0; i < a.length; i++)
			mp.put(Math.random(), i);

		// sort the map
		TreeMap<Double, Integer> tm = new TreeMap<Double, Integer>(mp);

		// go through the map and return the newly sorted array
		int[] b = new int[a.length];
		int j = 0;
		Iterator<Double> tmI = tm.keySet().iterator();
		while(tmI.hasNext())
		{
			b[j++] = a[tm.get(tmI.next()).intValue()];
		}

		return b;

	}
	
	public String buildModel()
	{
		mar = new MarianaInterface();
		String model = mar.build(trainSVM, valSVM, tstSVM);
		
		StringBuilder header = new StringBuilder();
		header.append("thesaurus: ");
		if(bow.thes != null)
			header.append(bow.thes.getThesaurusLine());
		else header.append("\n");
		
		header.append("terms: ");
		//header.append(bow.getTermLine());
		for(int i = 0; i < rankIndx.length; i++)
			header.append(String.format("%d:%s ", rankIndx[i], bow.getTerm(rankIndx[i])));
		header.append("\n");
		
		header.append("norm: ");
		for(int i = 0; i < rankIndx.length; i++)
			header.append(String.format("%d:%g ", rankIndx[i], terms.termNorm(rankIndx[i])));
		header.append("\n::LIBSVM::\n");
		
		header.append(model);
		
		return header.toString();
	}

}


